 # -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""

import pyqtgraph as pg
from ADoutputs import daq_AO, daq_DO
from pyqtgraph.Qt import QtGui, QtCore
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import (QPushButton, QWidget, QVBoxLayout, QLCDNumber, QProgressBar, QLabel)
import telnetlib
import os
from messages import show_warning

import scanFunctions 

#An example of a class
class anal_control_signal(QWidget):
    _registry = []
    
    def __init__(self, name, channel, out_minimum = -10, out_maximum = 10, out_step = 0.1):
        super().__init__()
        
        self._registry.append(self)
        
        self.name = name
        self.ch = channel
        
        self.initial_value = 0
        
        self.max = out_maximum
        self.min = out_minimum
        self.step = out_step
        self.siPrefix = True
        self.suffix = 'V'

        self.sb = pg.SpinBox(value = self.initial_value, bounds=[self.min, self.max], step = self.step, siPrefix = self.siPrefix, suffix = self.suffix)
        try:
            self.AO = daq_AO(self.ch)
        except:
            show_warning('Analog output %i not found. Daq might not be connected' % self.ch)
        
        
        
        
        
        self.scan_button = QPushButton('Start scan')
        
        self.ScanTimer = QTimer()
        self.ScanTimer.timeout.connect(self.scanEvent)
        self.scan_step = 0
        
        self.scanpBar = QProgressBar(self)
        self.scanLabelValue = QLabel(self)
        self.PbarTimer = QTimer(self)
        self.PbarTimer.timeout.connect(self.PbarEvent)

        self.scan_sb_start =  pg.SpinBox(value = 0, bounds=[self.min, self.max], step = self.step, siPrefix = self.siPrefix, suffix = self.suffix)
        self.scan_sb_stop =    pg.SpinBox(value = 0, bounds=[self.min, self.max], step = self.step, siPrefix = self.siPrefix, suffix = self.suffix)
        self.scan_sb_period = pg.SpinBox(value = 1, bounds=[0, 100], step = 0.1, siPrefix = True, suffix = 's')
        
        
        self.scan_button.clicked.connect(self.doScanAction)
        
        self.spin_boxes = [('sb', self.sb), ('scan_sb_start', self.scan_sb_start), ('scan_sb_stop', self.scan_sb_stop), ('scan_sb_period', self.scan_sb_period)]
        
        #set minimum height for spinboxes:
        self.sb.setMinimumHeight(22)
        self.scan_sb_period.setMinimumHeight(22)
        self.scan_sb_start.setMinimumHeight(22)
        self.scan_sb_stop.setMinimumHeight(22)
        
        
    def sb_analog_change(self):
        self.AO.set_out(self.sb.val)
        
    def set_name(self, name):
        self.name = name
    

        
    def set_siPrefix(self, value):
        self.siPrefix = value
    
    def set_sufix(self, value):
        self.sufix = value
    
        
    def connectSpinBoxesToDaq(self):
        print(self.name + 'Initial value: ', self.sb.val)
        self.sb_analog_change()
        self.sb.sigValueChanging.connect(self.sb_analog_change)
        
    
    def scanEvent(self):
         
        self.scan_step = next(self.scan_array_gen)
        self.AO.set_out(self.scan_step)
        

    def PbarEvent(self):
        try:
            self.scanpBar.setValue( 100 * float(self.scan_step-float(self.scan_sb_start.val))/float(self.scan_sb_stop.val-self.scan_sb_start.val)   )
            self.scanLabelValue.setText("%.2f" % self.scan_step + ' V')
        except ZeroDivisionError:
            self.doScanAction()
            show_warning('Oh dear, you might be trying to scan from 0 to 0. Plase never do that again.')
    
    def doScanAction(self):
        
                        
        if self.ScanTimer.isActive():
            self.ScanTimer.stop()
            self.scan_button.setText('Start scan')
            self.scan_button.setStyleSheet('background-color: None')
            self.PbarTimer.stop()
            self.sb.setValue(self.scan_step)
            self.sb.setEnabled(True)
            self.scan_sb_start.setEnabled(True)
            self.scan_sb_stop.setEnabled(True)
            
            
        else:
            
            dt = 5.0e-3
            
            first_value = float(self.sb.val)            
            self.scan_array = scanFunctions.create_scan_array(self.scan_sb_start.val, self.scan_sb_stop.val, self.scan_sb_period.val, first_value,dt)
            self.scan_array_gen = scanFunctions.yield_scan_array(self.scan_array)
            
            
            if first_value > float(self.scan_sb_stop.val) or first_value < float(self.scan_sb_start.val):
                details = 'Remember remember the fifth of November.' + '\n' + 'Also remember that scan starts from the value of the corresponding spin box'
                show_warning('Scan can not start outside scan range and scan start should be lower than scan stop', details_text= details)
            elif len(self.scan_array) < 5:
                show_warning('%i steps are too few, you need at least 5 steps' % (len(self.scan_array)) )
            else:
                self.sb.setEnabled(False)
                self.scan_sb_start.setEnabled(False)
                self.scan_sb_stop.setEnabled(False)
                self.scan_button.setStyleSheet("background-color: green")
                self.PbarTimer.start(dt*1e3)
                self.ScanTimer.start(dt*1e3)
                self.scan_button.setText('Scanning')
            
                
            
    
        
        

            
class digital_control_signal(QWidget):
    
    def __init__(self, name, channel):
        super().__init__()
        
        self.name = name
        self.ch = channel
        
        self.initial_value = 0
        try:
            self.DO = daq_DO(self.ch)
        except:
            show_warning('Digital output %i not found. Daq might not be connected' % self.ch)
            
        self.cb = QtGui.QCheckBox()
        self.cb.stateChanged.connect(self.whenChangeDigital)
        
    def set_name(self, name):
        self.name = name
    
    def set_initial_value(self, value):
        self.initial_value = value
    
    def whenChangeDigital(self):
        self.DO.set_out(not self.cb.isChecked())
        

class activation_button(QWidget):
    
    def __init__(self):
        QWidget.__init__(self)
        self.button = QPushButton('', self)
        self.button.clicked.connect(self.handleButton)
        self.button.setIcon(QtGui.QIcon(os.path.dirname(os.path.realpath(__file__)) + "\\snuffles1.png"))
        self.button.setIconSize(QtCore.QSize(48,48))
        layout = QVBoxLayout(self)
        layout.addWidget(self.button)

    def handleButton(self):
        self.button.setIcon(QtGui.QIcon(os.path.dirname(os.path.realpath(__file__))+ "\\snuffles.png"))

class burleighWM(QWidget):
        def __init__(self, layout, dock):
            super().__init__()
            
            self.dockWM = dock
            self.layoutWM = layout
            
            wavemeter_ok = False
            try:
                tn = telnetlib.Telnet(host = "wannanosaurus", port = 1234, timeout = 1)
                wavemeter_ok = True

            except:
                show_warning('Wannanosaurus not connected')

            
            
            
            if wavemeter_ok:
                tn.write(b'++eos 1\n')
    
                lcd = QLCDNumber()
            
                lcd.setDigitCount(9) 
                lcd.display("000000 00")
       
                def update_label():
                    tn.sock_avail()
                    tn.write(b'++read <cr>\n')
                    a = tn.read_very_eager().decode("utf-8")
                    #print(a)
                    try:
                        lcd.display(a[3:12])
                    except:
                        lcd.display("000000 00")
                    
                self.layoutWM.setGeometry(300, 300, 250, 150)
                
                self.layoutWM.timer = QTimer()
                self.layoutWM.timer.timeout.connect(update_label)
                self.layoutWM.timer.start(1000)  # every 10,000 milliseconds
                
                self.layoutWM.addWidget(lcd)
                self.dockWM.addWidget(self.layoutWM)
                    
     
      
                    
                